@@ -17,29 +17,19 @@ import copy
 import logging
 import socket
 
-from keystoneclient import adapter
-from keystoneclient import exceptions as ksc_exc
+from keystoneauth1 import adapter
+from keystoneauth1 import exceptions as ksa_exc
+import OpenSSL
 from oslo_utils import importutils
 from oslo_utils import netutils
 import requests
-try:
-    ProtocolError = requests.packages.urllib3.exceptions.ProtocolError
-except ImportError:
-    ProtocolError = requests.exceptions.ConnectionError
 import six
-from six.moves.urllib import parse
-import warnings
 
 try:
     import json
 except ImportError:
     import simplejson as json
 
-# Python 2.5 compat fix
-if not hasattr(parse, 'parse_qsl'):
-    import cgi
-    parse.parse_qsl = cgi.parse_qsl
-
 from oslo_utils import encodeutils
 
 from glanceclient.common import utils
@@ -50,6 +40,21 @@ osprofiler_web = importutils.try_import("osprofiler.web")
 LOG = logging.getLogger(__name__)
 USER_AGENT = 'python-glanceclient'
 CHUNKSIZE = 1024 * 64  # 64kB
+REQ_ID_HEADER = 'X-OpenStack-Request-ID'
+
+
+def encode_headers(headers):
+    """Encodes headers.
+
+    Note: This should be used right before
+    sending anything out.
+
+    :param headers: Headers to encode
+    :returns: Dictionary with encoded headers'
+              names and values
+    """
+    return dict((encodeutils.safe_encode(h), encodeutils.safe_encode(v))
+                for h, v in headers.items() if v is not None)
 
 
 class _BaseHTTPClient(object):
@@ -59,7 +64,7 @@ class _BaseHTTPClient(object):
         chunk = body
         while chunk:
             chunk = body.read(CHUNKSIZE)
-            if chunk == '':
+            if not chunk:
                 break
             yield chunk
 
@@ -89,7 +94,7 @@ class _BaseHTTPClient(object):
 
     def _handle_response(self, resp):
         if not resp.ok:
-            LOG.debug("Request returned failure status %s." % resp.status_code)
+            LOG.debug("Request returned failure status %s.", resp.status_code)
             raise exc.from_response(resp, resp.content)
         elif (resp.status_code == requests.codes.MULTIPLE_CHOICES and
               resp.request.path_url != '/versions'):
@@ -126,30 +131,20 @@ class HTTPClient(_BaseHTTPClient):
         self.identity_headers = kwargs.get('identity_headers')
         self.auth_token = kwargs.get('token')
         self.language_header = kwargs.get('language_header')
+        self.global_request_id = kwargs.get('global_request_id')
         if self.identity_headers:
-            if self.identity_headers.get('X-Auth-Token'):
-                self.auth_token = self.identity_headers.get('X-Auth-Token')
-                del self.identity_headers['X-Auth-Token']
+            self.auth_token = self.identity_headers.pop('X-Auth-Token',
+                                                        self.auth_token)
 
         self.session = requests.Session()
         self.session.headers["User-Agent"] = USER_AGENT
 
-        if self.auth_token:
-            self.session.headers["X-Auth-Token"] = self.auth_token
-
         if self.language_header:
             self.session.headers["Accept-Language"] = self.language_header
 
         self.timeout = float(kwargs.get('timeout', 600))
 
         if self.endpoint.startswith("https"):
-            compression = kwargs.get('ssl_compression', True)
-
-            if compression is False:
-                # Note: This is not seen by default. (python must be
-                # run with -Wd)
-                warnings.warn('The "ssl_compression" argument has been '
-                              'deprecated.', DeprecationWarning)
 
             if kwargs.get('insecure', False) is True:
                 self.session.verify = False
@@ -170,7 +165,7 @@ class HTTPClient(_BaseHTTPClient):
         headers = copy.deepcopy(headers)
         headers.update(self.session.headers)
 
-        for (key, value) in six.iteritems(headers):
+        for (key, value) in headers.items():
             header = '-H \'%s: %s\'' % utils.safe_header(key, value)
             curl.append(header)
 
@@ -206,20 +201,6 @@ class HTTPClient(_BaseHTTPClient):
         LOG.debug('\n'.join([encodeutils.safe_decode(x, errors='ignore')
                              for x in dump]))
 
-    @staticmethod
-    def encode_headers(headers):
-        """Encodes headers.
-
-        Note: This should be used right before
-        sending anything out.
-
-        :param headers: Headers to encode
-        :returns: Dictionary with encoded headers'
-                  names and values
-        """
-        return dict((encodeutils.safe_encode(h), encodeutils.safe_encode(v))
-                    for h, v in six.iteritems(headers) if v is not None)
-
     def _request(self, method, url, **kwargs):
         """Send an http request with the specified characteristics.
 
@@ -230,18 +211,25 @@ class HTTPClient(_BaseHTTPClient):
         headers = copy.deepcopy(kwargs.pop('headers', {}))
 
         if self.identity_headers:
-            for k, v in six.iteritems(self.identity_headers):
+            for k, v in self.identity_headers.items():
                 headers.setdefault(k, v)
 
         data = self._set_common_request_kwargs(headers, kwargs)
 
+        # add identity header to the request
+        if not headers.get('X-Auth-Token'):
+            headers['X-Auth-Token'] = self.auth_token
+
+        if self.global_request_id:
+            headers.setdefault(REQ_ID_HEADER, self.global_request_id)
+
         if osprofiler_web:
             headers.update(osprofiler_web.get_trace_id_headers())
 
         # Note(flaper87): Before letting headers / url fly,
         # they should be encoded otherwise httplib will
         # complain.
-        headers = self.encode_headers(headers)
+        headers = encode_headers(headers)
 
         if self.endpoint.endswith("/") or url.startswith("/"):
             conn_url = "%s%s" % (self.endpoint, url)
@@ -259,7 +247,7 @@ class HTTPClient(_BaseHTTPClient):
             message = ("Error communicating with %(url)s: %(e)s" %
                        dict(url=conn_url, e=e))
             raise exc.InvalidEndpoint(message=message)
-        except (requests.exceptions.ConnectionError, ProtocolError) as e:
+        except requests.exceptions.ConnectionError as e:
             message = ("Error finding address for %(url)s: %(e)s" %
                        dict(url=conn_url, e=e))
             raise exc.CommunicationError(message=message)
@@ -267,11 +255,25 @@ class HTTPClient(_BaseHTTPClient):
             message = "Error finding address for %s: %s" % (
                 self.endpoint_hostname, e)
             raise exc.InvalidEndpoint(message=message)
-        except (socket.error, socket.timeout) as e:
+        except (socket.error, socket.timeout, IOError) as e:
             endpoint = self.endpoint
             message = ("Error communicating with %(endpoint)s %(e)s" %
                        {'endpoint': endpoint, 'e': e})
             raise exc.CommunicationError(message=message)
+        except OpenSSL.SSL.Error as e:
+            message = ("SSL Error communicating with %(url)s: %(e)s" %
+                       {'url': conn_url, 'e': e})
+            raise exc.CommunicationError(message=message)
+
+        # log request-id for each api call
+        request_id = resp.headers.get('x-openstack-request-id')
+        if request_id:
+            LOG.debug('%(method)s call to image for '
+                      '%(url)s used request id '
+                      '%(response_request_id)s',
+                      {'method': resp.request.method,
+                       'url': resp.url,
+                       'response_request_id': request_id})
 
         resp, body_iter = self._handle_response(resp)
         self.log_http_response(resp)
@@ -312,26 +314,32 @@ class SessionClient(adapter.Adapter, _BaseHTTPClient):
     def __init__(self, session, **kwargs):
         kwargs.setdefault('user_agent', USER_AGENT)
         kwargs.setdefault('service_type', 'image')
+        self.global_request_id = kwargs.pop('global_request_id', None)
         super(SessionClient, self).__init__(session, **kwargs)
 
     def request(self, url, method, **kwargs):
         headers = kwargs.pop('headers', {})
+        if self.global_request_id:
+            headers.setdefault(REQ_ID_HEADER, self.global_request_id)
+
         kwargs['raise_exc'] = False
         data = self._set_common_request_kwargs(headers, kwargs)
-
         try:
-            resp = super(SessionClient, self).request(url,
-                                                      method,
-                                                      headers=headers,
-                                                      data=data,
-                                                      **kwargs)
-        except ksc_exc.RequestTimeout as e:
+            # NOTE(pumaranikar): To avoid bug #1641239, no modification of
+            # headers should be allowed after encode_headers() is called.
+            resp = super(SessionClient,
+                         self).request(url,
+                                       method,
+                                       headers=encode_headers(headers),
+                                       data=data,
+                                       **kwargs)
+        except ksa_exc.ConnectTimeout as e:
             conn_url = self.get_endpoint(auth=kwargs.get('auth'))
             conn_url = "%s/%s" % (conn_url.rstrip('/'), url.lstrip('/'))
             message = ("Error communicating with %(url)s %(e)s" %
                        dict(url=conn_url, e=e))
             raise exc.InvalidEndpoint(message=message)
-        except ksc_exc.ConnectionRefused as e:
+        except ksa_exc.ConnectFailure as e:
             conn_url = self.get_endpoint(auth=kwargs.get('auth'))
             conn_url = "%s/%s" % (conn_url.rstrip('/'), url.lstrip('/'))
             message = ("Error finding address for %(url)s: %(e)s" %
