@@ -14,10 +14,14 @@
 #    under the License.
 import functools
 import json
+import logging
+import uuid
 
-from keystoneclient.auth import token_endpoint
-from keystoneclient import session
+import fixtures
+from keystoneauth1 import session
+from keystoneauth1 import token_endpoint
 import mock
+from oslo_utils import encodeutils
 import requests
 from requests_mock.contrib import fixture
 import six
@@ -134,7 +138,7 @@ class TestClient(testtools.TestCase):
         http_client.get(path)
 
         headers = self.mock.last_request.headers
-        for k, v in six.iteritems(identity_headers):
+        for k, v in identity_headers.items():
             self.assertEqual(v, headers[k])
 
     def test_language_header_passed(self):
@@ -148,6 +152,18 @@ class TestClient(testtools.TestCase):
         headers = self.mock.last_request.headers
         self.assertEqual(kwargs['language_header'], headers['Accept-Language'])
 
+    def test_request_id_header_passed(self):
+        global_id = encodeutils.safe_encode("req-%s" % uuid.uuid4())
+        kwargs = {'global_request_id': global_id}
+        http_client = http.HTTPClient(self.endpoint, **kwargs)
+
+        path = '/v2/images/my-image'
+        self.mock.get(self.endpoint + path)
+        http_client.get(path)
+
+        headers = self.mock.last_request.headers
+        self.assertEqual(global_id, headers['X-OpenStack-Request-ID'])
+
     def test_language_header_not_passed_no_language(self):
         kwargs = {}
         http_client = http.HTTPClient(self.endpoint, **kwargs)
@@ -157,10 +173,10 @@ class TestClient(testtools.TestCase):
         http_client.get(path)
 
         headers = self.mock.last_request.headers
-        self.assertTrue('Accept-Language' not in headers)
+        self.assertNotIn('Accept-Language', headers)
 
     def test_connection_timeout(self):
-        """Should receive an InvalidEndpoint if connection timeout."""
+        """Verify a InvalidEndpoint is received if connection times out."""
         def cb(request, context):
             raise requests.exceptions.Timeout
 
@@ -172,11 +188,9 @@ class TestClient(testtools.TestCase):
         self.assertIn(self.endpoint, comm_err.message)
 
     def test_connection_refused(self):
-        """
+        """Verify a CommunicationError is received if connection is refused.
 
-        Should receive a CommunicationError if connection refused.
-        And the error should list the host and port that refused the
-        connection
+        The error should list the host and port that refused the connection.
         """
         def cb(request, context):
             raise requests.exceptions.ConnectionError()
@@ -201,15 +215,47 @@ class TestClient(testtools.TestCase):
         self.assertEqual(text, resp.text)
 
     def test_headers_encoding(self):
-        if not hasattr(self.client, 'encode_headers'):
-            self.skipTest('Cannot do header encoding check on SessionClient')
-
         value = u'ni\xf1o'
         headers = {"test": value, "none-val": None}
-        encoded = self.client.encode_headers(headers)
+        encoded = http.encode_headers(headers)
         self.assertEqual(b"ni\xc3\xb1o", encoded[b"test"])
         self.assertNotIn("none-val", encoded)
 
+    @mock.patch('keystoneauth1.adapter.Adapter.request')
+    def test_http_duplicate_content_type_headers(self, mock_ksarq):
+        """Test proper handling of Content-Type headers.
+
+        encode_headers() must be called as late as possible before a
+        request is sent. If this principle is violated, and if any
+        changes are made to the headers between encode_headers() and the
+        actual request (for instance a call to
+        _set_common_request_kwargs()), and if you're trying to set a
+        Content-Type that is not equal to application/octet-stream (the
+        default), it is entirely possible that you'll end up with two
+        Content-Type headers defined (yours plus
+        application/octet-stream). The request will go out the door with
+        only one of them chosen seemingly at random.
+
+        This situation only occurs in python3. This test will never fail
+        in python2.
+        """
+        path = "/v2/images/my-image"
+        headers = {
+            "Content-Type": "application/openstack-images-v2.1-json-patch"
+        }
+        data = '[{"value": "qcow2", "path": "/disk_format", "op": "replace"}]'
+        self.mock.patch(self.endpoint + path)
+        sess_http_client = self._create_session_client()
+        sess_http_client.patch(path, headers=headers, data=data)
+        # Pull out the headers with which Adapter.request was invoked
+        ksarqh = mock_ksarq.call_args[1]['headers']
+        # Only one Content-Type header (of any text-type)
+        self.assertEqual(1, [encodeutils.safe_decode(key)
+                             for key in ksarqh.keys()].count(u'Content-Type'))
+        # And it's the one we set
+        self.assertEqual(b"application/openstack-images-v2.1-json-patch",
+                         ksarqh[b"Content-Type"])
+
     def test_raw_request(self):
         """Verify the path being used for HTTP requests reflects accurately."""
         headers = {"Content-Type": "text/plain"}
@@ -234,7 +280,15 @@ class TestClient(testtools.TestCase):
     def test_get_connections_kwargs_http(self):
         endpoint = 'http://example.com:9292'
         test_client = http.HTTPClient(endpoint, token=u'adc123')
-        self.assertEqual(test_client.timeout, 600.0)
+        self.assertEqual(600.0, test_client.timeout)
+
+    def test__chunk_body_exact_size_chunk(self):
+        test_client = http._BaseHTTPClient()
+        bytestring = b'x' * http.CHUNKSIZE
+        data = six.BytesIO(bytestring)
+        chunk = list(test_client._chunk_body(data))
+        self.assertEqual(1, len(chunk))
+        self.assertEqual([bytestring], chunk)
 
     def test_http_chunked_request(self):
         text = "Ok"
@@ -267,7 +321,7 @@ class TestClient(testtools.TestCase):
                       headers={"Content-Type": "application/octet-stream"})
 
         resp, body = self.client.get(path)
-        self.assertTrue(isinstance(body, types.GeneratorType))
+        self.assertIsInstance(body, types.GeneratorType)
         self.assertEqual([data], list(body))
 
     @original_only
@@ -323,7 +377,7 @@ class TestClient(testtools.TestCase):
                         'LOG.debug called with no arguments')
 
         needles = {'key': key, 'cert': cert, 'cacert': cacert}
-        for option, value in six.iteritems(needles):
+        for option, value in needles.items():
             if value:
                 regex = ".*\s--%s\s+('%s'|%s).*" % (option, value, value)
                 self.assertThat(mock_log.call_args[0][0],
@@ -374,3 +428,42 @@ class TestClient(testtools.TestCase):
         self.assertThat(mock_log.call_args[0][0],
                         matchers.Not(matchers.MatchesRegex(token_regex)),
                         'token found in LOG.debug parameter')
+
+    def test_log_request_id_once(self):
+        logger = self.useFixture(fixtures.FakeLogger(level=logging.DEBUG))
+        data = "TEST"
+        path = '/v1/images/'
+        self.mock.get(self.endpoint + path, body=six.StringIO(data),
+                      headers={"Content-Type": "application/octet-stream",
+                               'x-openstack-request-id': "1234"})
+
+        resp, body = self.client.get(path)
+        self.assertIsInstance(body, types.GeneratorType)
+        self.assertEqual([data], list(body))
+        expected_log = ("GET call to image "
+                        "for http://example.com:9292/v1/images/ "
+                        "used request id 1234")
+        self.assertEqual(1, logger.output.count(expected_log))
+
+    def test_expired_token_has_changed(self):
+        # instantiate client with some token
+        fake_token = b'fake-token'
+        http_client = http.HTTPClient(self.endpoint,
+                                      token=fake_token)
+        path = '/v1/images/my-image'
+        self.mock.get(self.endpoint + path)
+        http_client.get(path)
+        headers = self.mock.last_request.headers
+        self.assertEqual(fake_token, headers['X-Auth-Token'])
+        # refresh the token
+        refreshed_token = b'refreshed-token'
+        http_client.auth_token = refreshed_token
+        http_client.get(path)
+        headers = self.mock.last_request.headers
+        self.assertEqual(refreshed_token, headers['X-Auth-Token'])
+        # regression check for bug 1448080
+        unicode_token = u'ni\xf1o'
+        http_client.auth_token = unicode_token
+        http_client.get(path)
+        headers = self.mock.last_request.headers
+        self.assertEqual(b'ni\xc3\xb1o', headers['X-Auth-Token'])
