@@ -15,6 +15,7 @@
 
 import testtools
 
+from glanceclient.tests.unit.v2 import base
 from glanceclient.tests import utils
 from glanceclient.v2 import image_members
 
@@ -80,12 +81,12 @@ class TestController(testtools.TestCase):
         super(TestController, self).setUp()
         self.api = utils.FakeAPI(data_fixtures)
         self.schema_api = utils.FakeSchemaAPI(schema_fixtures)
-        self.controller = image_members.Controller(self.api, self.schema_api)
+        self.controller = base.BaseController(self.api, self.schema_api,
+                                              image_members.Controller)
 
     def test_list_image_members(self):
         image_id = IMAGE
-        # NOTE(iccha): cast to list since the controller returns a generator
-        image_members = list(self.controller.list(image_id))
+        image_members = self.controller.list(image_id)
         self.assertEqual(IMAGE, image_members[0].image_id)
         self.assertEqual(MEMBER, image_members[0].member_id)
 
